@@ -8,7 +8,7 @@ from rest_framework import permissions
 
 from rest_access_policy import AccessPolicyException
 
-from .parsing import BoolAnd, BoolNot, BoolOr, ConditionOperand, boolOperand
+from .parsing import BoolAnd, BoolNot, BoolOr, ConditionOperand, BoolOperand
 
 
 class AnonymousUser(object):
@@ -19,8 +19,26 @@ class AnonymousUser(object):
         self.is_superuser = False
 
 
+class AccessEnforcement(object):
+    _action: str
+    _allowed: bool
+
+    def __init__(self, action: str, allowed: bool):
+        self._action = action
+        self._allowed = allowed
+
+    @property
+    def action(self) -> str:
+        return self._action
+
+    @property
+    def allowed(self) -> bool:
+        return self._allowed
+
+
 class AccessPolicy(permissions.BasePermission):
-    statements = []
+    statements: List[dict] = []
+    field_permissions: dict = {}
     id = None
     group_prefix = "group:"
     id_prefix = "id:"
@@ -32,7 +50,9 @@ class AccessPolicy(permissions.BasePermission):
         if len(statements) == 0:
             return False
 
-        return self._evaluate_statements(statements, request, view, action)
+        allowed = self._evaluate_statements(statements, request, view, action)
+        request.access_enforcement = AccessEnforcement(action=action, allowed=allowed)
+        return allowed
 
     def get_policy_statements(self, request, view) -> List[dict]:
         return self.statements
@@ -40,6 +60,7 @@ class AccessPolicy(permissions.BasePermission):
     def get_user_group_values(self, user) -> List[str]:
         if user.is_anonymous:
             return []
+
         prefetch_related_objects([user], "groups")
         return [g.name for g in user.groups.all()]
 
@@ -47,6 +68,10 @@ class AccessPolicy(permissions.BasePermission):
     def scope_queryset(cls, request, qs):
         return qs.none()
 
+    @classmethod
+    def scope_fields(cls, request, fields: dict, instance=None) -> dict:
+        return fields
+
     def _get_invoked_action(self, view) -> str:
         """
         If a CBV, the name of the method. If a regular function view,
@@ -59,6 +84,7 @@ class AccessPolicy(permissions.BasePermission):
 
         elif hasattr(view, "__class__"):
             return view.__class__.__name__
+
         raise AccessPolicyException("Could not determine action of request")
 
     def _evaluate_statements(self, statements: List[dict], request, view, action: str) -> bool:
@@ -101,7 +127,8 @@ class AccessPolicy(permissions.BasePermission):
 
         return statements
 
-    def _get_statements_matching_principal(self, request, statements: List[dict]) -> List[dict]:
+    @classmethod
+    def _get_statements_matching_principal(cls, request, statements: List[dict]) -> List[dict]:
         user = request.user or AnonymousUser()
         user_roles = None
         matched = []
@@ -120,14 +147,14 @@ class AccessPolicy(permissions.BasePermission):
                 found = True
             elif "anonymous" in principals and user.is_anonymous:
                 found = True
-            elif self.id_prefix + str(user.pk) in principals:
+            elif cls.id_prefix + str(user.pk) in principals:
                 found = True
             else:
                 if not user_roles:
-                    user_roles = self.get_user_group_values(user)
+                    user_roles = cls().get_user_group_values(user)
 
                 for user_role in user_roles:
-                    if self.group_prefix + user_role in principals:
+                    if cls.group_prefix + user_role in principals:
                         found = True
                         break
 
@@ -175,12 +202,13 @@ class AccessPolicy(permissions.BasePermission):
 
             fails = 0
 
+            boolOperand = BoolOperand()
+
             for condition in conditions:
                 if is_expression:
-                    ConditionOperand.check_condition_fn = lambda _, cond: self._check_condition(
-                        cond, request, view, action
-                    )
-                    boolOperand.setParseAction(ConditionOperand)
+                    check_cond_fn = lambda cond: self._check_condition(cond, request, view, action)
+
+                    boolOperand.setParseAction(lambda token: ConditionOperand(token, check_cond_fn))
 
                     boolExpr = infixNotation(
                         boolOperand,
